package net.wanji.web.task;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import net.wanji.web.common.entity.JsonViewObject;
import net.wanji.web.common.util.StringUtils;
import net.wanji.web.entity.TDeviceStatusInfo;
import net.wanji.web.mapper.TDeviceStatusMapper;
import net.wanji.web.util.HttpRestUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.Objects;

import static net.wanji.web.util.HttpRestUtil.buildHeader;

/**
 * @author duanruiming
 * @date 2023/01/03 10:43
 */
@Component
public class SignalStatusTask {

    private static final Logger LOGGER = LoggerFactory.getLogger(SignalStatusTask.class);

    @Autowired
    private HttpRestUtil httpRestUtil;

    @Value("${utcServiceUrl}")
    private String utcServiceUrl;
    @Value("${syncDeviceSatusUrn}")
    private String syncDeviceSatusUrl;

    @Autowired
    private TDeviceStatusMapper tDeviceStatusMapper;

    @Scheduled(fixedRate = 5 * 60 * 1000)
    public void syncSignalStatus() {
        // 远程通过UTC服务获取信号机状态
        Map<String, String> header = buildHeader();
        JsonViewObject jsonViewObject = httpRestUtil.doPost(utcServiceUrl.concat(syncDeviceSatusUrl), header, "", JsonViewObject.class);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            LOGGER.error("定时任务同步信号机设备状态远程utcService调用异常，url:".concat(utcServiceUrl.concat("/signalStatus/runningStatusAlarm")));
            return;
        }
        List<Map<String, Object>> content = (List<Map<String, Object>>) jsonViewObject.getContent();
        Map<String, Object> result = content.get(0);
        String signalId = (String) result.get("signalId");
        Integer currentSignalStatus = (Integer) result.get("status");

        // 当前服务数据库状态
        LambdaQueryWrapper<TDeviceStatusInfo> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotEmpty(signalId)) {
            lambdaQueryWrapper.eq(TDeviceStatusInfo::getCode, signalId);
            TDeviceStatusInfo tDeviceStatusInfo = tDeviceStatusMapper.selectOne(lambdaQueryWrapper);
            if (Objects.nonNull(tDeviceStatusInfo) && currentSignalStatus != tDeviceStatusInfo.getStatus()) {
                tDeviceStatusInfo.setStatus(currentSignalStatus);
                tDeviceStatusMapper.updateById(tDeviceStatusInfo);
            }
        }
    }
}
